@@ -7,21 +7,101 @@ from operator import attrgetter
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import FieldError
+from django.db import connection
 from django.db.models import (
     Avg, Count, F, Max, Q, StdDev, Sum, Value, Variance,
 )
-from django.test import TestCase, skipUnlessDBFeature
+from django.test import TestCase, skipUnlessAnyDBFeature, skipUnlessDBFeature
 from django.test.utils import Approximate
 from django.utils import six
 
 from .models import (
     Alfa, Author, Book, Bravo, Charlie, Clues, Entries, HardbackBook, ItemTag,
-    Publisher, SelfRefFK, WithManualPK,
+    Publisher, SelfRefFK, Store, WithManualPK,
 )
 
 
 class AggregationTests(TestCase):
-    fixtures = ["aggregation_regress.json"]
+
+    @classmethod
+    def setUpTestData(cls):
+        cls.a1 = Author.objects.create(name='Adrian Holovaty', age=34)
+        cls.a2 = Author.objects.create(name='Jacob Kaplan-Moss', age=35)
+        cls.a3 = Author.objects.create(name='Brad Dayley', age=45)
+        cls.a4 = Author.objects.create(name='James Bennett', age=29)
+        cls.a5 = Author.objects.create(name='Jeffrey Forcier', age=37)
+        cls.a6 = Author.objects.create(name='Paul Bissex', age=29)
+        cls.a7 = Author.objects.create(name='Wesley J. Chun', age=25)
+        cls.a8 = Author.objects.create(name='Peter Norvig', age=57)
+        cls.a9 = Author.objects.create(name='Stuart Russell', age=46)
+        cls.a1.friends.add(cls.a2, cls.a4)
+        cls.a2.friends.add(cls.a1, cls.a7)
+        cls.a4.friends.add(cls.a1)
+        cls.a5.friends.add(cls.a6, cls.a7)
+        cls.a6.friends.add(cls.a5, cls.a7)
+        cls.a7.friends.add(cls.a2, cls.a5, cls.a6)
+        cls.a8.friends.add(cls.a9)
+        cls.a9.friends.add(cls.a8)
+
+        cls.p1 = Publisher.objects.create(name='Apress', num_awards=3)
+        cls.p2 = Publisher.objects.create(name='Sams', num_awards=1)
+        cls.p3 = Publisher.objects.create(name='Prentice Hall', num_awards=7)
+        cls.p4 = Publisher.objects.create(name='Morgan Kaufmann', num_awards=9)
+        cls.p5 = Publisher.objects.create(name="Jonno's House of Books", num_awards=0)
+
+        cls.b1 = Book.objects.create(
+            isbn='159059725', name='The Definitive Guide to Django: Web Development Done Right',
+            pages=447, rating=4.5, price=Decimal('30.00'), contact=cls.a1, publisher=cls.p1,
+            pubdate=datetime.date(2007, 12, 6)
+        )
+        cls.b2 = Book.objects.create(
+            isbn='067232959', name='Sams Teach Yourself Django in 24 Hours',
+            pages=528, rating=3.0, price=Decimal('23.09'), contact=cls.a3, publisher=cls.p2,
+            pubdate=datetime.date(2008, 3, 3)
+        )
+        cls.b3 = Book.objects.create(
+            isbn='159059996', name='Practical Django Projects',
+            pages=300, rating=4.0, price=Decimal('29.69'), contact=cls.a4, publisher=cls.p1,
+            pubdate=datetime.date(2008, 6, 23)
+        )
+        cls.b4 = Book.objects.create(
+            isbn='013235613', name='Python Web Development with Django',
+            pages=350, rating=4.0, price=Decimal('29.69'), contact=cls.a5, publisher=cls.p3,
+            pubdate=datetime.date(2008, 11, 3)
+        )
+        cls.b5 = HardbackBook.objects.create(
+            isbn='013790395', name='Artificial Intelligence: A Modern Approach',
+            pages=1132, rating=4.0, price=Decimal('82.80'), contact=cls.a8, publisher=cls.p3,
+            pubdate=datetime.date(1995, 1, 15), weight=4.5)
+        cls.b6 = HardbackBook.objects.create(
+            isbn='155860191', name='Paradigms of Artificial Intelligence Programming: Case Studies in Common Lisp',
+            pages=946, rating=5.0, price=Decimal('75.00'), contact=cls.a8, publisher=cls.p4,
+            pubdate=datetime.date(1991, 10, 15), weight=3.7)
+        cls.b1.authors.add(cls.a1, cls.a2)
+        cls.b2.authors.add(cls.a3)
+        cls.b3.authors.add(cls.a4)
+        cls.b4.authors.add(cls.a5, cls.a6, cls.a7)
+        cls.b5.authors.add(cls.a8, cls.a9)
+        cls.b6.authors.add(cls.a8)
+
+        s1 = Store.objects.create(
+            name='Amazon.com',
+            original_opening=datetime.datetime(1994, 4, 23, 9, 17, 42),
+            friday_night_closing=datetime.time(23, 59, 59)
+        )
+        s2 = Store.objects.create(
+            name='Books.com',
+            original_opening=datetime.datetime(2001, 3, 15, 11, 23, 37),
+            friday_night_closing=datetime.time(23, 59, 59)
+        )
+        s3 = Store.objects.create(
+            name="Mamma and Pappa's Books",
+            original_opening=datetime.datetime(1945, 4, 25, 16, 24, 14),
+            friday_night_closing=datetime.time(21, 30)
+        )
+        s1.books.add(cls.b1, cls.b2, cls.b3, cls.b4, cls.b5, cls.b6)
+        s2.books.add(cls.b1, cls.b3, cls.b5, cls.b6)
+        s3.books.add(cls.b3, cls.b4, cls.b6)
 
     def assertObjectAttrs(self, obj, **kwargs):
         for attr, value in six.iteritems(kwargs):
@@ -123,84 +203,90 @@ class AggregationTests(TestCase):
 
     def test_annotation(self):
         # Annotations get combined with extra select clauses
-        obj = Book.objects.annotate(mean_auth_age=Avg("authors__age")).extra(select={"manufacture_cost": "price * .5"}).get(pk=2)
-        self.assertObjectAttrs(obj,
-            contact_id=3,
-            id=2,
+        obj = Book.objects.annotate(mean_auth_age=Avg("authors__age")).extra(
+            select={"manufacture_cost": "price * .5"}).get(pk=self.b2.pk)
+        self.assertObjectAttrs(
+            obj,
+            contact_id=self.a3.id,
             isbn='067232959',
             mean_auth_age=45.0,
             name='Sams Teach Yourself Django in 24 Hours',
             pages=528,
             price=Decimal("23.09"),
             pubdate=datetime.date(2008, 3, 3),
-            publisher_id=2,
+            publisher_id=self.p2.id,
             rating=3.0
         )
         # Different DB backends return different types for the extra select computation
         self.assertIn(obj.manufacture_cost, (11.545, Decimal('11.545')))
 
         # Order of the annotate/extra in the query doesn't matter
-        obj = Book.objects.extra(select={'manufacture_cost': 'price * .5'}).annotate(mean_auth_age=Avg('authors__age')).get(pk=2)
-        self.assertObjectAttrs(obj,
-            contact_id=3,
-            id=2,
+        obj = Book.objects.extra(select={'manufacture_cost': 'price * .5'}).annotate(
+            mean_auth_age=Avg('authors__age')).get(pk=self.b2.pk)
+        self.assertObjectAttrs(
+            obj,
+            contact_id=self.a3.id,
             isbn='067232959',
             mean_auth_age=45.0,
             name='Sams Teach Yourself Django in 24 Hours',
             pages=528,
             price=Decimal("23.09"),
             pubdate=datetime.date(2008, 3, 3),
-            publisher_id=2,
+            publisher_id=self.p2.id,
             rating=3.0
         )
         # Different DB backends return different types for the extra select computation
         self.assertIn(obj.manufacture_cost, (11.545, Decimal('11.545')))
 
         # Values queries can be combined with annotate and extra
-        obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(select={'manufacture_cost': 'price * .5'}).values().get(pk=2)
+        obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(
+            select={'manufacture_cost': 'price * .5'}).values().get(pk=self.b2.pk)
         manufacture_cost = obj['manufacture_cost']
         self.assertIn(manufacture_cost, (11.545, Decimal('11.545')))
         del obj['manufacture_cost']
         self.assertEqual(obj, {
-            "contact_id": 3,
-            "id": 2,
-            "isbn": "067232959",
-            "mean_auth_age": 45.0,
-            "name": "Sams Teach Yourself Django in 24 Hours",
-            "pages": 528,
-            "price": Decimal("23.09"),
-            "pubdate": datetime.date(2008, 3, 3),
-            "publisher_id": 2,
-            "rating": 3.0,
-        })
-
-        # The order of the (empty) values, annotate and extra clauses doesn't
-        # matter
-        obj = Book.objects.values().annotate(mean_auth_age=Avg('authors__age')).extra(select={'manufacture_cost': 'price * .5'}).get(pk=2)
-        manufacture_cost = obj['manufacture_cost']
-        self.assertIn(manufacture_cost, (11.545, Decimal('11.545')))
-        del obj['manufacture_cost']
-        self.assertEqual(obj, {
-            'contact_id': 3,
-            'id': 2,
+            'id': self.b2.id,
+            'contact_id': self.a3.id,
             'isbn': '067232959',
             'mean_auth_age': 45.0,
             'name': 'Sams Teach Yourself Django in 24 Hours',
             'pages': 528,
-            'price': Decimal("23.09"),
+            'price': Decimal('23.09'),
             'pubdate': datetime.date(2008, 3, 3),
-            'publisher_id': 2,
+            'publisher_id': self.p2.id,
+            'rating': 3.0,
+        })
+
+        # The order of the (empty) values, annotate and extra clauses doesn't
+        # matter
+        obj = Book.objects.values().annotate(mean_auth_age=Avg('authors__age')).extra(
+            select={'manufacture_cost': 'price * .5'}).get(pk=self.b2.pk)
+        manufacture_cost = obj['manufacture_cost']
+        self.assertIn(manufacture_cost, (11.545, Decimal('11.545')))
+        del obj['manufacture_cost']
+        self.assertEqual(obj, {
+            'id': self.b2.id,
+            'contact_id': self.a3.id,
+            'isbn': '067232959',
+            'mean_auth_age': 45.0,
+            'name': 'Sams Teach Yourself Django in 24 Hours',
+            'pages': 528,
+            'price': Decimal('23.09'),
+            'pubdate': datetime.date(2008, 3, 3),
+            'publisher_id': self.p2.id,
             'rating': 3.0
         })
 
         # If the annotation precedes the values clause, it won't be included
         # unless it is explicitly named
-        obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(select={'price_per_page': 'price / pages'}).values('name').get(pk=1)
+        obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(
+            select={'price_per_page': 'price / pages'}).values('name').get(pk=self.b1.pk)
         self.assertEqual(obj, {
             "name": 'The Definitive Guide to Django: Web Development Done Right',
         })
 
-        obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(select={'price_per_page': 'price / pages'}).values('name', 'mean_auth_age').get(pk=1)
+        obj = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(
+            select={'price_per_page': 'price / pages'}).values('name', 'mean_auth_age').get(pk=self.b1.pk)
         self.assertEqual(obj, {
             'mean_auth_age': 34.5,
             'name': 'The Definitive Guide to Django: Web Development Done Right',
@@ -218,7 +304,8 @@ class AggregationTests(TestCase):
 
         # The annotations are added to values output if values() precedes
         # annotate()
-        obj = Book.objects.values('name').annotate(mean_auth_age=Avg('authors__age')).extra(select={'price_per_page': 'price / pages'}).get(pk=1)
+        obj = Book.objects.values('name').annotate(mean_auth_age=Avg('authors__age')).extra(
+            select={'price_per_page': 'price / pages'}).get(pk=self.b1.pk)
         self.assertEqual(obj, {
             'mean_auth_age': 34.5,
             'name': 'The Definitive Guide to Django: Web Development Done Right',
@@ -232,7 +319,13 @@ class AggregationTests(TestCase):
         )
 
         # Check that consecutive calls to annotate accumulate in the query
-        qs = Book.objects.values('price').annotate(oldest=Max('authors__age')).order_by('oldest', 'price').annotate(Max('publisher__num_awards'))
+        qs = (
+            Book.objects
+            .values('price')
+            .annotate(oldest=Max('authors__age'))
+            .order_by('oldest', 'price')
+            .annotate(Max('publisher__num_awards'))
+        )
         self.assertQuerysetEqual(
             qs, [
                 {'price': Decimal("30"), 'oldest': 35, 'publisher__num_awards__max': 3},
@@ -247,7 +340,12 @@ class AggregationTests(TestCase):
     def test_aggrate_annotation(self):
         # Aggregates can be composed over annotations.
         # The return type is derived from the composed aggregate
-        vals = Book.objects.all().annotate(num_authors=Count('authors__id')).aggregate(Max('pages'), Max('price'), Sum('num_authors'), Avg('num_authors'))
+        vals = (
+            Book.objects
+            .all()
+            .annotate(num_authors=Count('authors__id'))
+            .aggregate(Max('pages'), Max('price'), Sum('num_authors'), Avg('num_authors'))
+        )
         self.assertEqual(vals, {
             'num_authors__sum': 10,
             'num_authors__avg': Approximate(1.666, places=2),
@@ -262,6 +360,20 @@ class AggregationTests(TestCase):
             {'c__max': 3}
         )
 
+    def test_decimal_aggregate_annotation_filter(self):
+        """
+        Filtering on an aggregate annotation with Decimal values should work.
+        Requires special handling on SQLite (#18247).
+        """
+        self.assertEqual(
+            len(Author.objects.annotate(sum=Sum('book_contact_set__price')).filter(sum__gt=Decimal(40))),
+            1
+        )
+        self.assertEqual(
+            len(Author.objects.annotate(sum=Sum('book_contact_set__price')).filter(sum__lte=Decimal(40))),
+            4
+        )
+
     def test_field_error(self):
         # Bad field requests in aggregates are caught and reported
         self.assertRaises(
@@ -309,17 +421,18 @@ class AggregationTests(TestCase):
         )
 
         # Regression for #10064: select_related() plays nice with aggregates
-        obj = Book.objects.select_related('publisher').annotate(num_authors=Count('authors')).values()[0]
+        obj = Book.objects.select_related('publisher').annotate(
+            num_authors=Count('authors')).values().get(isbn='013790395')
         self.assertEqual(obj, {
-            'contact_id': 8,
-            'id': 5,
+            'contact_id': self.a8.id,
+            'id': self.b5.id,
             'isbn': '013790395',
             'name': 'Artificial Intelligence: A Modern Approach',
             'num_authors': 2,
             'pages': 1132,
             'price': Decimal("82.8"),
             'pubdate': datetime.date(1995, 1, 15),
-            'publisher_id': 3,
+            'publisher_id': self.p3.id,
             'rating': 4.0,
         })
 
@@ -339,18 +452,34 @@ class AggregationTests(TestCase):
         )
 
         self.assertEqual(
-            len(Book.objects.annotate(num_authors=Count('authors')).filter(num_authors__lt=3).exclude(num_authors__lt=2)),
+            len(
+                Book.objects
+                .annotate(num_authors=Count('authors'))
+                .filter(num_authors__lt=3)
+                .exclude(num_authors__lt=2)
+            ),
             2
         )
         self.assertEqual(
-            len(Book.objects.annotate(num_authors=Count('authors')).exclude(num_authors__lt=2).filter(num_authors__lt=3)),
+            len(
+                Book.objects
+                .annotate(num_authors=Count('authors'))
+                .exclude(num_authors__lt=2)
+                .filter(num_authors__lt=3)
+            ),
             2
         )
 
     def test_aggregate_fexpr(self):
         # Aggregates can be used with F() expressions
         # ... where the F() is pushed into the HAVING clause
-        qs = Publisher.objects.annotate(num_books=Count('book')).filter(num_books__lt=F('num_awards') / 2).order_by('name').values('name', 'num_books', 'num_awards')
+        qs = (
+            Publisher.objects
+            .annotate(num_books=Count('book'))
+            .filter(num_books__lt=F('num_awards') / 2)
+            .order_by('name')
+            .values('name', 'num_books', 'num_awards')
+        )
         self.assertQuerysetEqual(
             qs, [
                 {'num_books': 1, 'name': 'Morgan Kaufmann', 'num_awards': 9},
@@ -359,7 +488,13 @@ class AggregationTests(TestCase):
             lambda p: p,
         )
 
-        qs = Publisher.objects.annotate(num_books=Count('book')).exclude(num_books__lt=F('num_awards') / 2).order_by('name').values('name', 'num_books', 'num_awards')
+        qs = (
+            Publisher.objects
+            .annotate(num_books=Count('book'))
+            .exclude(num_books__lt=F('num_awards') / 2)
+            .order_by('name')
+            .values('name', 'num_books', 'num_awards')
+        )
         self.assertQuerysetEqual(
             qs, [
                 {'num_books': 2, 'name': 'Apress', 'num_awards': 3},
@@ -370,7 +505,13 @@ class AggregationTests(TestCase):
         )
 
         # ... and where the F() references an aggregate
-        qs = Publisher.objects.annotate(num_books=Count('book')).filter(num_awards__gt=2 * F('num_books')).order_by('name').values('name', 'num_books', 'num_awards')
+        qs = (
+            Publisher.objects
+            .annotate(num_books=Count('book'))
+            .filter(num_awards__gt=2 * F('num_books'))
+            .order_by('name')
+            .values('name', 'num_books', 'num_awards')
+        )
         self.assertQuerysetEqual(
             qs, [
                 {'num_books': 1, 'name': 'Morgan Kaufmann', 'num_awards': 9},
@@ -379,7 +520,13 @@ class AggregationTests(TestCase):
             lambda p: p,
         )
 
-        qs = Publisher.objects.annotate(num_books=Count('book')).exclude(num_books__lt=F('num_awards') / 2).order_by('name').values('name', 'num_books', 'num_awards')
+        qs = (
+            Publisher.objects
+            .annotate(num_books=Count('book'))
+            .exclude(num_books__lt=F('num_awards') / 2)
+            .order_by('name')
+            .values('name', 'num_books', 'num_awards')
+        )
         self.assertQuerysetEqual(
             qs, [
                 {'num_books': 2, 'name': 'Apress', 'num_awards': 3},
@@ -391,7 +538,11 @@ class AggregationTests(TestCase):
 
     def test_db_col_table(self):
         # Tests on fields with non-default table and column names.
-        qs = Clues.objects.values('EntryID__Entry').annotate(Appearances=Count('EntryID'), Distinct_Clues=Count('Clue', distinct=True))
+        qs = (
+            Clues.objects
+            .values('EntryID__Entry')
+            .annotate(Appearances=Count('EntryID'), Distinct_Clues=Count('Clue', distinct=True))
+        )
         self.assertQuerysetEqual(qs, [])
 
         qs = Entries.objects.annotate(clue_count=Count('clues__ID'))
@@ -416,17 +567,45 @@ class AggregationTests(TestCase):
             0
         )
 
-        vals = Book.objects.filter(id__in=[]).aggregate(num_authors=Count('authors'), avg_authors=Avg('authors'), max_authors=Max('authors'), max_price=Max('price'), max_rating=Max('rating'))
+        vals = (
+            Book.objects
+            .filter(id__in=[])
+            .aggregate(
+                num_authors=Count('authors'),
+                avg_authors=Avg('authors'),
+                max_authors=Max('authors'),
+                max_price=Max('price'),
+                max_rating=Max('rating'),
+            )
+        )
         self.assertEqual(
             vals,
             {'max_authors': None, 'max_rating': None, 'num_authors': 0, 'avg_authors': None, 'max_price': None}
         )
 
-        qs = Publisher.objects.filter(pk=5).annotate(num_authors=Count('book__authors'), avg_authors=Avg('book__authors'), max_authors=Max('book__authors'), max_price=Max('book__price'), max_rating=Max('book__rating')).values()
+        qs = (
+            Publisher.objects
+            .filter(name="Jonno's House of Books")
+            .annotate(
+                num_authors=Count('book__authors'),
+                avg_authors=Avg('book__authors'),
+                max_authors=Max('book__authors'),
+                max_price=Max('book__price'),
+                max_rating=Max('book__rating'),
+            ).values()
+        )
         self.assertQuerysetEqual(
-            qs, [
-                {'max_authors': None, 'name': "Jonno's House of Books", 'num_awards': 0, 'max_price': None, 'num_authors': 0, 'max_rating': None, 'id': 5, 'avg_authors': None}
-            ],
+            qs,
+            [{
+                'max_authors': None,
+                'name': "Jonno's House of Books",
+                'num_awards': 0,
+                'max_price': None,
+                'num_authors': 0,
+                'max_rating': None,
+                'id': self.p5.id,
+                'avg_authors': None,
+            }],
             lambda p: p
         )
 
@@ -449,10 +628,16 @@ class AggregationTests(TestCase):
         # Regression for #10127 - Empty select_related() works with annotate
         qs = Book.objects.filter(rating__lt=4.5).select_related().annotate(Avg('authors__age'))
         self.assertQuerysetEqual(
-            qs, [
+            qs,
+            [
                 ('Artificial Intelligence: A Modern Approach', 51.5, 'Prentice Hall', 'Peter Norvig'),
                 ('Practical Django Projects', 29.0, 'Apress', 'James Bennett'),
-                ('Python Web Development with Django', Approximate(30.333, places=2), 'Prentice Hall', 'Jeffrey Forcier'),
+                (
+                    'Python Web Development with Django',
+                    Approximate(30.333, places=2),
+                    'Prentice Hall',
+                    'Jeffrey Forcier',
+                ),
                 ('Sams Teach Yourself Django in 24 Hours', 45.0, 'Sams', 'Brad Dayley')
             ],
             lambda b: (b.name, b.authors__age__avg, b.publisher.name, b.contact.name)
@@ -463,28 +648,40 @@ class AggregationTests(TestCase):
         qs = Book.objects.extra(select={'pub': 'publisher_id'}).values('pub').annotate(Count('id')).order_by('pub')
         self.assertQuerysetEqual(
             qs, [
-                {'pub': 1, 'id__count': 2},
-                {'pub': 2, 'id__count': 1},
-                {'pub': 3, 'id__count': 2},
-                {'pub': 4, 'id__count': 1}
+                {'pub': self.b1.id, 'id__count': 2},
+                {'pub': self.b2.id, 'id__count': 1},
+                {'pub': self.b3.id, 'id__count': 2},
+                {'pub': self.b4.id, 'id__count': 1}
             ],
             lambda b: b
         )
 
-        qs = Book.objects.extra(select={'pub': 'publisher_id', 'foo': 'pages'}).values('pub').annotate(Count('id')).order_by('pub')
+        qs = (
+            Book.objects
+            .extra(select={'pub': 'publisher_id', 'foo': 'pages'})
+            .values('pub')
+            .annotate(Count('id'))
+            .order_by('pub')
+        )
         self.assertQuerysetEqual(
             qs, [
-                {'pub': 1, 'id__count': 2},
-                {'pub': 2, 'id__count': 1},
-                {'pub': 3, 'id__count': 2},
-                {'pub': 4, 'id__count': 1}
+                {'pub': self.p1.id, 'id__count': 2},
+                {'pub': self.p2.id, 'id__count': 1},
+                {'pub': self.p3.id, 'id__count': 2},
+                {'pub': self.p4.id, 'id__count': 1}
             ],
             lambda b: b
         )
 
         # Regression for #10182 - Queries with aggregate calls are correctly
         # realiased when used in a subquery
-        ids = Book.objects.filter(pages__gt=100).annotate(n_authors=Count('authors')).filter(n_authors__gt=2).order_by('n_authors')
+        ids = (
+            Book.objects
+            .filter(pages__gt=100)
+            .annotate(n_authors=Count('authors'))
+            .filter(n_authors__gt=2)
+            .order_by('n_authors')
+        )
         self.assertQuerysetEqual(
             Book.objects.filter(id__in=ids), [
                 "Python Web Development with Django",
@@ -501,21 +698,27 @@ class AggregationTests(TestCase):
 
     def test_duplicate_alias(self):
         # Regression for #11256 - duplicating a default alias raises ValueError.
-        self.assertRaises(ValueError, Book.objects.all().annotate, Avg('authors__age'), authors__age__avg=Avg('authors__age'))
+        self.assertRaises(
+            ValueError,
+            Book.objects.all().annotate,
+            Avg('authors__age'), authors__age__avg=Avg('authors__age')
+        )
 
     def test_field_name_conflict(self):
-        # Regression for #11256 - providing an aggregate name that conflicts with a field name on the model raises ValueError
+        # Regression for #11256 - providing an aggregate name
+        # that conflicts with a field name on the model raises ValueError
         self.assertRaises(ValueError, Author.objects.annotate, age=Avg('friends__age'))
 
     def test_m2m_name_conflict(self):
-        # Regression for #11256 - providing an aggregate name that conflicts with an m2m name on the model raises ValueError
+        # Regression for #11256 - providing an aggregate name
+        # that conflicts with an m2m name on the model raises ValueError
         self.assertRaises(ValueError, Author.objects.annotate, friends=Count('friends'))
 
     def test_values_queryset_non_conflict(self):
         # Regression for #14707 -- If you're using a values query set, some potential conflicts are avoided.
 
         # age is a field on Author, so it shouldn't be allowed as an aggregate.
-        # But age isn't included in the ValuesQuerySet, so it is.
+        # But age isn't included in values(), so it is.
         results = Author.objects.values('name').annotate(age=Count('book_contact_set')).order_by('name')
         self.assertEqual(len(results), 9)
         self.assertEqual(results[0]['name'], 'Adrian Holovaty')
@@ -534,7 +737,8 @@ class AggregationTests(TestCase):
         self.assertEqual(results[0]['friends'], 2)
 
     def test_reverse_relation_name_conflict(self):
-        # Regression for #11256 - providing an aggregate name that conflicts with a reverse-related name on the model raises ValueError
+        # Regression for #11256 - providing an aggregate name
+        # that conflicts with a reverse-related name on the model raises ValueError
         self.assertRaises(ValueError, Author.objects.annotate, book_contact_set=Avg('friends__age'))
 
     def test_pickle(self):
@@ -580,7 +784,13 @@ class AggregationTests(TestCase):
 
         # Regression for #10290 - extra selects with parameters can be used for
         # grouping.
-        qs = Book.objects.annotate(mean_auth_age=Avg('authors__age')).extra(select={'sheets': '(pages + %s) / %s'}, select_params=[1, 2]).order_by('sheets').values('sheets')
+        qs = (
+            Book.objects
+            .annotate(mean_auth_age=Avg('authors__age'))
+            .extra(select={'sheets': '(pages + %s) / %s'}, select_params=[1, 2])
+            .order_by('sheets')
+            .values('sheets')
+        )
         self.assertQuerysetEqual(
             qs, [
                 150,
@@ -664,18 +874,26 @@ class AggregationTests(TestCase):
 
         qs = HardbackBook.objects.annotate(n_authors=Count('book_ptr__authors')).values('name', 'n_authors')
         self.assertQuerysetEqual(
-            qs, [
+            qs,
+            [
                 {'n_authors': 2, 'name': 'Artificial Intelligence: A Modern Approach'},
-                {'n_authors': 1, 'name': 'Paradigms of Artificial Intelligence Programming: Case Studies in Common Lisp'}
+                {
+                    'n_authors': 1,
+                    'name': 'Paradigms of Artificial Intelligence Programming: Case Studies in Common Lisp'
+                }
             ],
             lambda h: h
         )
 
         qs = HardbackBook.objects.annotate(n_authors=Count('authors')).values('name', 'n_authors')
         self.assertQuerysetEqual(
-            qs, [
+            qs,
+            [
                 {'n_authors': 2, 'name': 'Artificial Intelligence: A Modern Approach'},
-                {'n_authors': 1, 'name': 'Paradigms of Artificial Intelligence Programming: Case Studies in Common Lisp'}
+                {
+                    'n_authors': 1,
+                    'name': 'Paradigms of Artificial Intelligence Programming: Case Studies in Common Lisp'
+                }
             ],
             lambda h: h,
         )
@@ -769,11 +987,17 @@ class AggregationTests(TestCase):
             attrgetter("name")
         )
 
-        qs = Book.objects.annotate(n_authors=Count("authors")).filter(
-            Q(name="The Definitive Guide to Django: Web Development Done Right") | (Q(name="Artificial Intelligence: A Modern Approach") & Q(n_authors=3))
+        qs = (
+            Book.objects
+            .annotate(n_authors=Count("authors"))
+            .filter(
+                Q(name="The Definitive Guide to Django: Web Development Done Right")
+                | (Q(name="Artificial Intelligence: A Modern Approach") & Q(n_authors=3))
+            )
         )
         self.assertQuerysetEqual(
-            qs, [
+            qs,
+            [
                 "The Definitive Guide to Django: Web Development Done Right",
             ],
             attrgetter("name")
@@ -798,12 +1022,15 @@ class AggregationTests(TestCase):
             rating_sum=Sum("book__rating"),
             book_count=Count("book")
         ).filter(
-            Q(pk__lt=F("book_count")) | Q(rating_sum=None)
-        ).order_by("pk")
+            Q(rating_sum__gt=F("book_count")) | Q(rating_sum=None)
+        ).order_by("num_awards")
         self.assertQuerysetEqual(
             qs, [
-                "Apress",
                 "Jonno's House of Books",
+                "Sams",
+                "Apress",
+                "Prentice Hall",
+                "Morgan Kaufmann"
             ],
             attrgetter("name")
         )
@@ -909,6 +1136,11 @@ class AggregationTests(TestCase):
             ['Peter Norvig'],
             lambda b: b.name
         )
+        # Referencing the auto-generated name in an aggregate() also works.
+        self.assertEqual(
+            Author.objects.annotate(Count('book')).aggregate(Max('book__count')),
+            {'book__count__max': 2}
+        )
 
     def test_annotate_joins(self):
         """
@@ -921,7 +1153,7 @@ class AggregationTests(TestCase):
         # Check that the query executes without problems.
         self.assertEqual(len(qs.exclude(publisher=-1)), 6)
 
-    @skipUnlessDBFeature("allows_group_by_pk")
+    @skipUnlessAnyDBFeature('allows_group_by_pk', 'allows_group_by_selected_pks')
     def test_aggregate_duplicate_columns(self):
         # Regression test for #17144
 
@@ -951,7 +1183,7 @@ class AggregationTests(TestCase):
             ]
         )
 
-    @skipUnlessDBFeature("allows_group_by_pk")
+    @skipUnlessAnyDBFeature('allows_group_by_pk', 'allows_group_by_selected_pks')
     def test_aggregate_duplicate_columns_only(self):
         # Works with only() too.
         results = Author.objects.only('id', 'name').annotate(num_contacts=Count('book_contact_set'))
@@ -977,13 +1209,14 @@ class AggregationTests(TestCase):
             ]
         )
 
-    @skipUnlessDBFeature("allows_group_by_pk")
+    @skipUnlessAnyDBFeature('allows_group_by_pk', 'allows_group_by_selected_pks')
     def test_aggregate_duplicate_columns_select_related(self):
         # And select_related()
         results = Book.objects.select_related('contact').annotate(
             num_authors=Count('authors'))
         _, _, grouping = results.query.get_compiler(using='default').pre_sql_setup()
-        self.assertEqual(len(grouping), 1)
+        # In the case of `group_by_selected_pks` we also group by contact.id because of the select_related.
+        self.assertEqual(len(grouping), 1 if connection.features.allows_group_by_pk else 2)
         self.assertIn('id', grouping[0][0])
         self.assertNotIn('name', grouping[0][0])
         self.assertNotIn('contact', grouping[0][0])
@@ -1131,6 +1364,19 @@ class AggregationTests(TestCase):
             'select__avg': Approximate(1.666, places=2),
         })
 
+    def test_annotate_on_relation(self):
+        book = Book.objects.annotate(avg_price=Avg('price'), publisher_name=F('publisher__name')).get(pk=self.b1.pk)
+        self.assertEqual(book.avg_price, 30.00)
+        self.assertEqual(book.publisher_name, "Apress")
+
+    def test_aggregate_on_relation(self):
+        # A query with an existing annotation aggregation on a relation should
+        # succeed.
+        qs = Book.objects.annotate(avg_price=Avg('price')).aggregate(
+            publisher_awards=Sum('publisher__num_awards')
+        )
+        self.assertEqual(qs['publisher_awards'], 30)
+
     def test_annotate_distinct_aggregate(self):
         # There are three books with rating of 4.0 and two of the books have
         # the same price. Hence, the distinct removes one rating of 4.0
