@@ -8,11 +8,12 @@ from django.db.models import (
     BooleanField, CharField, Count, DateTimeField, ExpressionWrapper, F, Func,
     IntegerField, Sum, Value,
 )
-from django.test import TestCase
+from django.db.models.functions import Lower
+from django.test import TestCase, skipUnlessDBFeature
 from django.utils import six
 
 from .models import (
-    Author, Book, Company, DepartmentStore, Employee, Store, Ticket,
+    Author, Book, Company, DepartmentStore, Employee, Publisher, Store, Ticket,
 )
 
 
@@ -29,7 +30,88 @@ def cxOracle_py3_bug(func):
 
 
 class NonAggregateAnnotationTestCase(TestCase):
-    fixtures = ["annotations.json"]
+
+    @classmethod
+    def setUpTestData(cls):
+        cls.a1 = Author.objects.create(name='Adrian Holovaty', age=34)
+        cls.a2 = Author.objects.create(name='Jacob Kaplan-Moss', age=35)
+        cls.a3 = Author.objects.create(name='Brad Dayley', age=45)
+        cls.a4 = Author.objects.create(name='James Bennett', age=29)
+        cls.a5 = Author.objects.create(name='Jeffrey Forcier', age=37)
+        cls.a6 = Author.objects.create(name='Paul Bissex', age=29)
+        cls.a7 = Author.objects.create(name='Wesley J. Chun', age=25)
+        cls.a8 = Author.objects.create(name='Peter Norvig', age=57)
+        cls.a9 = Author.objects.create(name='Stuart Russell', age=46)
+        cls.a1.friends.add(cls.a2, cls.a4)
+        cls.a2.friends.add(cls.a1, cls.a7)
+        cls.a4.friends.add(cls.a1)
+        cls.a5.friends.add(cls.a6, cls.a7)
+        cls.a6.friends.add(cls.a5, cls.a7)
+        cls.a7.friends.add(cls.a2, cls.a5, cls.a6)
+        cls.a8.friends.add(cls.a9)
+        cls.a9.friends.add(cls.a8)
+
+        cls.p1 = Publisher.objects.create(name='Apress', num_awards=3)
+        cls.p2 = Publisher.objects.create(name='Sams', num_awards=1)
+        cls.p3 = Publisher.objects.create(name='Prentice Hall', num_awards=7)
+        cls.p4 = Publisher.objects.create(name='Morgan Kaufmann', num_awards=9)
+        cls.p5 = Publisher.objects.create(name="Jonno's House of Books", num_awards=0)
+
+        cls.b1 = Book.objects.create(
+            isbn='159059725', name='The Definitive Guide to Django: Web Development Done Right',
+            pages=447, rating=4.5, price=Decimal('30.00'), contact=cls.a1, publisher=cls.p1,
+            pubdate=datetime.date(2007, 12, 6)
+        )
+        cls.b2 = Book.objects.create(
+            isbn='067232959', name='Sams Teach Yourself Django in 24 Hours',
+            pages=528, rating=3.0, price=Decimal('23.09'), contact=cls.a3, publisher=cls.p2,
+            pubdate=datetime.date(2008, 3, 3)
+        )
+        cls.b3 = Book.objects.create(
+            isbn='159059996', name='Practical Django Projects',
+            pages=300, rating=4.0, price=Decimal('29.69'), contact=cls.a4, publisher=cls.p1,
+            pubdate=datetime.date(2008, 6, 23)
+        )
+        cls.b4 = Book.objects.create(
+            isbn='013235613', name='Python Web Development with Django',
+            pages=350, rating=4.0, price=Decimal('29.69'), contact=cls.a5, publisher=cls.p3,
+            pubdate=datetime.date(2008, 11, 3)
+        )
+        cls.b5 = Book.objects.create(
+            isbn='013790395', name='Artificial Intelligence: A Modern Approach',
+            pages=1132, rating=4.0, price=Decimal('82.80'), contact=cls.a8, publisher=cls.p3,
+            pubdate=datetime.date(1995, 1, 15)
+        )
+        cls.b6 = Book.objects.create(
+            isbn='155860191', name='Paradigms of Artificial Intelligence Programming: Case Studies in Common Lisp',
+            pages=946, rating=5.0, price=Decimal('75.00'), contact=cls.a8, publisher=cls.p4,
+            pubdate=datetime.date(1991, 10, 15)
+        )
+        cls.b1.authors.add(cls.a1, cls.a2)
+        cls.b2.authors.add(cls.a3)
+        cls.b3.authors.add(cls.a4)
+        cls.b4.authors.add(cls.a5, cls.a6, cls.a7)
+        cls.b5.authors.add(cls.a8, cls.a9)
+        cls.b6.authors.add(cls.a8)
+
+        s1 = Store.objects.create(
+            name='Amazon.com',
+            original_opening=datetime.datetime(1994, 4, 23, 9, 17, 42),
+            friday_night_closing=datetime.time(23, 59, 59)
+        )
+        s2 = Store.objects.create(
+            name='Books.com',
+            original_opening=datetime.datetime(2001, 3, 15, 11, 23, 37),
+            friday_night_closing=datetime.time(23, 59, 59)
+        )
+        s3 = Store.objects.create(
+            name="Mamma and Pappa's Books",
+            original_opening=datetime.datetime(1945, 4, 25, 16, 24, 14),
+            friday_night_closing=datetime.time(21, 30)
+        )
+        s1.books.add(cls.b1, cls.b2, cls.b3, cls.b4, cls.b5, cls.b6)
+        s2.books.add(cls.b1, cls.b3, cls.b5, cls.b6)
+        s3.books.add(cls.b3, cls.b4, cls.b6)
 
     def test_basic_annotation(self):
         books = Book.objects.annotate(
@@ -59,7 +141,7 @@ class NonAggregateAnnotationTestCase(TestCase):
         self.assertEqual(t.expires, expires)
 
     def test_mixed_type_annotation_numbers(self):
-        test = Book.objects.get(isbn='159059725')
+        test = self.b1
         b = Book.objects.annotate(
             combined=ExpressionWrapper(F('pages') + F('rating'), output_field=IntegerField())
         ).get(isbn=test.isbn)
@@ -79,6 +161,40 @@ class NonAggregateAnnotationTestCase(TestCase):
         other_agg = Author.objects.aggregate(age_sum=Sum('age'))
         self.assertEqual(agg['otherage_sum'], other_agg['age_sum'])
 
+    @skipUnlessDBFeature('can_distinct_on_fields')
+    def test_distinct_on_with_annotation(self):
+        store = Store.objects.create(
+            name='test store',
+            original_opening=datetime.datetime.now(),
+            friday_night_closing=datetime.time(21, 00, 00),
+        )
+        names = [
+            'Theodore Roosevelt',
+            'Eleanor Roosevelt',
+            'Franklin Roosevelt',
+            'Ned Stark',
+            'Catelyn Stark',
+        ]
+        for name in names:
+            Employee.objects.create(
+                store=store,
+                first_name=name.split()[0],
+                last_name=name.split()[1],
+                age=30, salary=2000,
+            )
+
+        people = Employee.objects.annotate(
+            name_lower=Lower('last_name'),
+        ).distinct('name_lower')
+
+        self.assertEqual(set(p.last_name for p in people), {'Stark', 'Roosevelt'})
+        self.assertEqual(len(people), 2)
+
+        people2 = Employee.objects.annotate(
+            test_alias=F('store__name'),
+        ).distinct('test_alias')
+        self.assertEqual(len(people2), 1)
+
     def test_filter_annotation(self):
         books = Book.objects.annotate(
             is_book=Value(1, output_field=IntegerField())
@@ -114,22 +230,21 @@ class NonAggregateAnnotationTestCase(TestCase):
             ).filter(sum_rating=F('nope')))
 
     def test_combined_annotation_commutative(self):
-        b1 = Book.objects.get(isbn='159059725')
-        book1 = Book.objects.annotate(adjusted_rating=F('rating') + 2).get(pk=b1.pk)
-        book2 = Book.objects.annotate(adjusted_rating=2 + F('rating')).get(pk=b1.pk)
+        book1 = Book.objects.annotate(adjusted_rating=F('rating') + 2).get(pk=self.b1.pk)
+        book2 = Book.objects.annotate(adjusted_rating=2 + F('rating')).get(pk=self.b1.pk)
         self.assertEqual(book1.adjusted_rating, book2.adjusted_rating)
-        book1 = Book.objects.annotate(adjusted_rating=F('rating') + None).get(pk=b1.pk)
-        book2 = Book.objects.annotate(adjusted_rating=None + F('rating')).get(pk=b1.pk)
+        book1 = Book.objects.annotate(adjusted_rating=F('rating') + None).get(pk=self.b1.pk)
+        book2 = Book.objects.annotate(adjusted_rating=None + F('rating')).get(pk=self.b1.pk)
         self.assertEqual(book1.adjusted_rating, book2.adjusted_rating)
 
     def test_update_with_annotation(self):
-        book_preupdate = Book.objects.get(pk=2)
+        book_preupdate = Book.objects.get(pk=self.b2.pk)
         Book.objects.annotate(other_rating=F('rating') - 1).update(rating=F('other_rating'))
-        book_postupdate = Book.objects.get(pk=2)
+        book_postupdate = Book.objects.get(pk=self.b2.pk)
         self.assertEqual(book_preupdate.rating - 1, book_postupdate.rating)
 
     def test_annotation_with_m2m(self):
-        books = Book.objects.annotate(author_age=F('authors__age')).filter(pk=1).order_by('author_age')
+        books = Book.objects.annotate(author_age=F('authors__age')).filter(pk=self.b1.pk).order_by('author_age')
         self.assertEqual(books[0].author_age, 34)
         self.assertEqual(books[1].author_age, 35)
 
@@ -155,7 +270,7 @@ class NonAggregateAnnotationTestCase(TestCase):
         """
         # annotate references a field in values()
         qs = Book.objects.values('rating').annotate(other_rating=F('rating') - 1)
-        book = qs.get(pk=1)
+        book = qs.get(pk=self.b1.pk)
         self.assertEqual(book['rating'] - 1, book['other_rating'])
 
         # filter refs the annotated value
