@@ -8,6 +8,3 @@ with RemoveGroupsAndPermissions():
     class UUIDUser(AbstractUser):
         """A user with a UUID as primary key"""
         id = models.UUIDField(default=uuid.uuid4, primary_key=True)
-
-        class Meta:
-            app_label = 'auth'
