@@ -11,7 +11,7 @@ from django.db.models.aggregates import (
     Avg, Count, Max, Min, StdDev, Sum, Variance,
 )
 from django.db.models.expressions import (
-    F, Case, Col, Date, DateTime, ExpressionWrapper, Func, OrderBy, Random,
+    Case, Col, Date, DateTime, ExpressionWrapper, F, Func, OrderBy, Random,
     RawSQL, Ref, Value, When,
 )
 from django.db.models.functions import (
@@ -261,6 +261,32 @@ class BasicExpressionsTests(TestCase):
         test_gmbh = Company.objects.get(pk=test_gmbh.pk)
         self.assertEqual(test_gmbh.num_employees, 36)
 
+    def test_new_object_save(self):
+        # We should be able to use Funcs when inserting new data
+        test_co = Company(
+            name=Lower(Value("UPPER")), num_employees=32, num_chairs=1,
+            ceo=Employee.objects.create(firstname="Just", lastname="Doit", salary=30),
+        )
+        test_co.save()
+        test_co.refresh_from_db()
+        self.assertEqual(test_co.name, "upper")
+
+    def test_new_object_create(self):
+        test_co = Company.objects.create(
+            name=Lower(Value("UPPER")), num_employees=32, num_chairs=1,
+            ceo=Employee.objects.create(firstname="Just", lastname="Doit", salary=30),
+        )
+        test_co.refresh_from_db()
+        self.assertEqual(test_co.name, "upper")
+
+    def test_object_create_with_aggregate(self):
+        # Aggregates are not allowed when inserting new data
+        with self.assertRaisesMessage(FieldError, 'Aggregate functions are not allowed in this query'):
+            Company.objects.create(
+                name='Company', num_employees=Max(Value(1)), num_chairs=1,
+                ceo=Employee.objects.create(firstname="Just", lastname="Doit", salary=30),
+            )
+
     def test_object_update_fk(self):
         # F expressions cannot be used to update attributes which are foreign
         # keys, or attributes which involve joins.
@@ -284,7 +310,22 @@ class BasicExpressionsTests(TestCase):
             ceo=test_gmbh.ceo
         )
         acme.num_employees = F("num_employees") + 16
-        self.assertRaises(TypeError, acme.save)
+        msg = (
+            'Failed to insert expression "Col(expressions_company, '
+            'expressions.Company.num_employees) + Value(16)" on '
+            'expressions.Company.num_employees. F() expressions can only be '
+            'used to update, not to insert.'
+        )
+        self.assertRaisesMessage(ValueError, msg, acme.save)
+
+        acme.num_employees = 12
+        acme.name = Lower(F('name'))
+        msg = (
+            'Failed to insert expression "Lower(Col(expressions_company, '
+            'expressions.Company.name))" on expressions.Company.name. F() '
+            'expressions can only be used to update, not to insert.'
+        )
+        self.assertRaisesMessage(ValueError, msg, acme.save)
 
     def test_ticket_11722_iexact_lookup(self):
         Employee.objects.create(firstname="John", lastname="Doe")
@@ -708,11 +749,12 @@ class FTimeDeltaTests(TestCase):
         self.assertEqual(q1, q2)
 
     def test_query_clone(self):
-        # Ticket #21643
+        # Ticket #21643 - Crash when compiling query more than once
         qs = Experiment.objects.filter(end__lt=F('start') + datetime.timedelta(hours=1))
         qs2 = qs.all()
         list(qs)
         list(qs2)
+        # Intentionally no assert
 
     def test_delta_add(self):
         for i in range(len(self.deltas)):
