@@ -1,6 +1,7 @@
-from django.contrib.gis.db import models
 from django.utils.encoding import python_2_unicode_compatible
 
+from ..models import models
+
 
 @python_2_unicode_compatible
 class NamedModel(models.Model):
@@ -10,6 +11,7 @@ class NamedModel(models.Model):
 
     class Meta:
         abstract = True
+        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return self.name
@@ -45,10 +47,9 @@ class Polygon3D(NamedModel):
 
 class SimpleModel(models.Model):
 
-    objects = models.GeoManager()
-
     class Meta:
         abstract = True
+        required_db_features = ['gis_enabled']
 
 
 class Point2D(SimpleModel):
