@@ -1,4 +1,4 @@
-from django.contrib.gis.db import models
+from ..models import models
 
 
 class AllOGRFields(models.Model):
@@ -13,4 +13,14 @@ class AllOGRFields(models.Model):
     geom = models.PolygonField()
     point = models.PointField()
 
-    objects = models.GeoManager()
+    class Meta:
+        required_db_features = ['gis_enabled']
+
+
+class Fields3D(models.Model):
+    point = models.PointField(dim=3)
+    line = models.LineStringField(dim=3)
+    poly = models.PolygonField(dim=3)
+
+    class Meta:
+        required_db_features = ['gis_enabled']
