@@ -2,19 +2,22 @@
 from __future__ import unicode_literals
 
 import os
+import socket
 import unittest
+import warnings
 from unittest import skipUnless
 
 from django.conf import settings
 from django.contrib.gis.geoip import HAS_GEOIP
-from django.contrib.gis.geos import HAS_GEOS
+from django.contrib.gis.geos import HAS_GEOS, GEOSGeometry
+from django.test import ignore_warnings
 from django.utils import six
+from django.utils.deprecation import RemovedInDjango20Warning
+from django.utils.encoding import force_text
 
 if HAS_GEOIP:
     from django.contrib.gis.geoip import GeoIP, GeoIPException
-
-if HAS_GEOS:
-    from django.contrib.gis.geos import GEOSGeometry
+    from django.contrib.gis.geoip.prototypes import GeoIP_lib_version
 
 
 # Note: Requires use of both the GeoIP country and city datasets.
@@ -25,9 +28,21 @@ if HAS_GEOS:
 
 @skipUnless(HAS_GEOIP and getattr(settings, "GEOIP_PATH", None),
     "GeoIP is required along with the GEOIP_PATH setting.")
+@ignore_warnings(category=RemovedInDjango20Warning)
 class GeoIPTest(unittest.TestCase):
-    addr = '162.242.220.127'
-    fqdn = 'www.djangoproject.com'
+    addr = '128.249.1.1'
+    fqdn = 'tmc.edu'
+
+    def _is_dns_available(self, domain):
+        # Naive check to see if there is DNS available to use.
+        # Used to conditionally skip fqdn geoip checks.
+        # See #25407 for details.
+        ErrClass = socket.error if six.PY2 else OSError
+        try:
+            socket.gethostbyname(domain)
+            return True
+        except ErrClass:
+            return False
 
     def test01_init(self):
         "Testing GeoIP initialization."
@@ -73,11 +88,14 @@ class GeoIPTest(unittest.TestCase):
         "Testing GeoIP country querying methods."
         g = GeoIP(city='<foo>')
 
-        for query in (self.fqdn, self.addr):
+        queries = [self.addr]
+        if self._is_dns_available(self.fqdn):
+            queries.append(self.fqdn)
+        for query in queries:
             for func in (g.country_code, g.country_code_by_addr, g.country_code_by_name):
-                self.assertEqual('US', func(query))
+                self.assertEqual('US', func(query), 'Failed for func %s and query %s' % (func, query))
             for func in (g.country_name, g.country_name_by_addr, g.country_name_by_name):
-                self.assertEqual('United States', func(query))
+                self.assertEqual('United States', func(query), 'Failed for func %s and query %s' % (func, query))
             self.assertEqual({'country_code': 'US', 'country_name': 'United States'},
                              g.country(query))
 
@@ -86,7 +104,10 @@ class GeoIPTest(unittest.TestCase):
         "Testing GeoIP city querying methods."
         g = GeoIP(country='<foo>')
 
-        for query in (self.fqdn, self.addr):
+        queries = [self.addr]
+        if self._is_dns_available(self.fqdn):
+            queries.append(self.fqdn)
+        for query in queries:
             # Country queries should still work.
             for func in (g.country_code, g.country_code_by_addr, g.country_code_by_name):
                 self.assertEqual('US', func(query))
@@ -98,23 +119,52 @@ class GeoIPTest(unittest.TestCase):
             # City information dictionary.
             d = g.city(query)
             self.assertEqual('USA', d['country_code3'])
-            self.assertEqual('San Antonio', d['city'])
+            self.assertEqual('Houston', d['city'])
             self.assertEqual('TX', d['region'])
-            self.assertEqual(210, d['area_code'])
+            self.assertEqual(713, d['area_code'])
             geom = g.geos(query)
             self.assertIsInstance(geom, GEOSGeometry)
-            lon, lat = (-98, 29)
+            lon, lat = (-95.4010, 29.7079)
             lat_lon = g.lat_lon(query)
             lat_lon = (lat_lon[1], lat_lon[0])
             for tup in (geom.tuple, g.coords(query), g.lon_lat(query), lat_lon):
-                self.assertAlmostEqual(lon, tup[0], 0)
-                self.assertAlmostEqual(lat, tup[1], 0)
+                self.assertAlmostEqual(lon, tup[0], 4)
+                self.assertAlmostEqual(lat, tup[1], 4)
 
     def test05_unicode_response(self):
         "Testing that GeoIP strings are properly encoded, see #16553."
         g = GeoIP()
-        d = g.city("messe-duesseldorf.com")
-        self.assertEqual('Düsseldorf', d['city'])
+        fqdn = "duesseldorf.de"
+        if self._is_dns_available(fqdn):
+            d = g.city(fqdn)
+            self.assertEqual('Ratingen', d['city'])
         d = g.country('200.26.205.1')
         # Some databases have only unaccented countries
         self.assertIn(d['country_name'], ('Curaçao', 'Curacao'))
+
+    def test_deprecation_warning(self):
+        with warnings.catch_warnings(record=True) as warns:
+            warnings.simplefilter('always')
+            GeoIP()
+
+        self.assertEqual(len(warns), 1)
+        msg = str(warns[0].message)
+        self.assertIn('django.contrib.gis.geoip is deprecated', msg)
+
+    def test_repr(self):
+        path = settings.GEOIP_PATH
+        g = GeoIP(path=path)
+        country_path = g._country_file
+        city_path = g._city_file
+        if GeoIP_lib_version:
+            expected = '<GeoIP [v%(version)s] _country_file="%(country)s", _city_file="%(city)s">' % {
+                'version': force_text(GeoIP_lib_version()),
+                'country': country_path,
+                'city': city_path,
+            }
+        else:
+            expected = '<GeoIP _country_file="%(country)s", _city_file="%(city)s">' % {
+                'country': country_path,
+                'city': city_path,
+            }
+        self.assertEqual(repr(g), expected)
