@@ -1,11 +1,13 @@
 import sys
 
 from django.conf import settings
+from django.core.exceptions import MiddlewareNotUsed
 from django.core.signals import got_request_exception
 from django.http import HttpResponse
+from django.template import engines
 from django.template.response import TemplateResponse
-from django.template import Template
-from django.test import TestCase, override_settings
+from django.test import RequestFactory, SimpleTestCase, override_settings
+from django.test.utils import patch_logger
 
 
 class TestException(Exception):
@@ -61,7 +63,8 @@ class ResponseMiddleware(TestMiddleware):
 class TemplateResponseMiddleware(TestMiddleware):
     def process_template_response(self, request, response):
         super(TemplateResponseMiddleware, self).process_template_response(request, response)
-        return TemplateResponse(request, Template('Template Response Middleware'))
+        template = engines['django'].from_string('Template Response Middleware')
+        return TemplateResponse(request, template)
 
 
 class ExceptionMiddleware(TestMiddleware):
@@ -101,8 +104,19 @@ class BadExceptionMiddleware(TestMiddleware):
         raise TestException('Test Exception Exception')
 
 
-class BaseMiddlewareExceptionTest(TestCase):
-    urls = 'middleware_exceptions.urls'
+# Sample middlewares that omit to return an HttpResonse
+class NoTemplateResponseMiddleware(TestMiddleware):
+    def process_template_response(self, request, response):
+        super(NoTemplateResponseMiddleware, self).process_template_response(request, response)
+
+
+class NoResponseMiddleware(TestMiddleware):
+    def process_response(self, request, response):
+        super(NoResponseMiddleware, self).process_response(request, response)
+
+
+@override_settings(ROOT_URLCONF='middleware_exceptions.urls')
+class BaseMiddlewareExceptionTest(SimpleTestCase):
 
     def setUp(self):
         self.exceptions = []
@@ -381,10 +395,13 @@ class MiddlewareTests(BaseMiddlewareExceptionTest):
         self._add_middleware(post_middleware)
         self._add_middleware(middleware)
         self._add_middleware(pre_middleware)
-        self.assert_exceptions_handled('/middleware_exceptions/null_view/', [
-            "The view middleware_exceptions.views.null_view didn't return an HttpResponse object. It returned None instead.",
-        ],
-            ValueError())
+        self.assert_exceptions_handled(
+            '/middleware_exceptions/null_view/', [
+                "The view middleware_exceptions.views.null_view didn't return "
+                "an HttpResponse object. It returned None instead."
+            ],
+            ValueError()
+        )
 
         # Check that the right middleware methods have been invoked
         self.assert_middleware_usage(pre_middleware, True, True, False, True, False)
@@ -398,10 +415,13 @@ class MiddlewareTests(BaseMiddlewareExceptionTest):
         self._add_middleware(post_middleware)
         self._add_middleware(middleware)
         self._add_middleware(pre_middleware)
-        self.assert_exceptions_handled('/middleware_exceptions/null_view/', [
-            "The view middleware_exceptions.views.null_view didn't return an HttpResponse object. It returned None instead."
-        ],
-            ValueError())
+        self.assert_exceptions_handled(
+            '/middleware_exceptions/null_view/', [
+                "The view middleware_exceptions.views.null_view didn't return "
+                "an HttpResponse object. It returned None instead."
+            ],
+            ValueError()
+        )
 
         # Check that the right middleware methods have been invoked
         self.assert_middleware_usage(pre_middleware, True, True, False, True, False)
@@ -472,6 +492,16 @@ class MiddlewareTests(BaseMiddlewareExceptionTest):
         # Check that the right middleware methods have been invoked
         self.assert_middleware_usage(middleware, True, True, True, True, False)
 
+    @override_settings(
+        MIDDLEWARE_CLASSES=['middleware_exceptions.middleware.ProcessExceptionMiddleware'],
+    )
+    def test_exception_in_render_passed_to_process_exception(self):
+        # Repopulate the list of middlewares since it's already been populated
+        # by setUp() before the MIDDLEWARE_CLASSES setting got overridden
+        self.client.handler.load_middleware()
+        response = self.client.get('/middleware_exceptions/exception_in_render/')
+        self.assertEqual(response.content, b'Exception caught')
+
 
 class BadMiddlewareTests(BaseMiddlewareExceptionTest):
 
@@ -510,7 +540,10 @@ class BadMiddlewareTests(BaseMiddlewareExceptionTest):
         self._add_middleware(post_middleware)
         self._add_middleware(bad_middleware)
         self._add_middleware(pre_middleware)
-        self.assert_exceptions_handled('/middleware_exceptions/template_response/', ['Test Template Response Exception'])
+        self.assert_exceptions_handled(
+            '/middleware_exceptions/template_response/',
+            ['Test Template Response Exception']
+        )
 
         # Check that the right middleware methods have been invoked
         self.assert_middleware_usage(pre_middleware, True, True, False, True, False)
@@ -692,10 +725,13 @@ class BadMiddlewareTests(BaseMiddlewareExceptionTest):
         self._add_middleware(post_middleware)
         self._add_middleware(bad_middleware)
         self._add_middleware(pre_middleware)
-        self.assert_exceptions_handled('/middleware_exceptions/null_view/', [
-            "The view middleware_exceptions.views.null_view didn't return an HttpResponse object. It returned None instead.",
-            'Test Response Exception'
-        ])
+        self.assert_exceptions_handled(
+            '/middleware_exceptions/null_view/', [
+                "The view middleware_exceptions.views.null_view didn't return "
+                "an HttpResponse object. It returned None instead.",
+                'Test Response Exception'
+            ]
+        )
 
         # Check that the right middleware methods have been invoked
         self.assert_middleware_usage(pre_middleware, True, True, False, False, False)
@@ -709,10 +745,13 @@ class BadMiddlewareTests(BaseMiddlewareExceptionTest):
         self._add_middleware(post_middleware)
         self._add_middleware(bad_middleware)
         self._add_middleware(pre_middleware)
-        self.assert_exceptions_handled('/middleware_exceptions/null_view/', [
-            "The view middleware_exceptions.views.null_view didn't return an HttpResponse object. It returned None instead."
-        ],
-            ValueError())
+        self.assert_exceptions_handled(
+            '/middleware_exceptions/null_view/', [
+                "The view middleware_exceptions.views.null_view didn't return "
+                "an HttpResponse object. It returned None instead."
+            ],
+            ValueError()
+        )
 
         # Check that the right middleware methods have been invoked
         self.assert_middleware_usage(pre_middleware, True, True, False, True, False)
@@ -775,12 +814,48 @@ class BadMiddlewareTests(BaseMiddlewareExceptionTest):
         self.assert_middleware_usage(bad_middleware, True, True, False, True, True)
         self.assert_middleware_usage(post_middleware, True, True, False, True, True)
 
+    def test_process_response_no_response_middleware(self):
+        pre_middleware = TestMiddleware()
+        middleware = NoResponseMiddleware()
+        post_middleware = TestMiddleware()
+        self._add_middleware(post_middleware)
+        self._add_middleware(middleware)
+        self._add_middleware(pre_middleware)
+        self.assert_exceptions_handled('/middleware_exceptions/view/', [
+            "NoResponseMiddleware.process_response didn't return an HttpResponse object. It returned None instead."
+        ],
+            ValueError())
+
+        # Check that the right middleware methods have been invoked
+        self.assert_middleware_usage(pre_middleware, True, True, False, False, False)
+        self.assert_middleware_usage(middleware, True, True, False, True, False)
+        self.assert_middleware_usage(post_middleware, True, True, False, True, False)
+
+    def test_process_template_response_no_response_middleware(self):
+        pre_middleware = TestMiddleware()
+        middleware = NoTemplateResponseMiddleware()
+        post_middleware = TestMiddleware()
+        self._add_middleware(post_middleware)
+        self._add_middleware(middleware)
+        self._add_middleware(pre_middleware)
+        self.assert_exceptions_handled(
+            '/middleware_exceptions/template_response/', [
+                "NoTemplateResponseMiddleware.process_template_response didn't "
+                "return an HttpResponse object. It returned None instead."
+            ],
+            ValueError()
+        )
+
+        # Check that the right middleware methods have been invoked
+        self.assert_middleware_usage(pre_middleware, True, True, False, True, False)
+        self.assert_middleware_usage(middleware, True, True, True, True, False)
+        self.assert_middleware_usage(post_middleware, True, True, True, True, False)
 
 _missing = object()
 
 
-class RootUrlconfTests(TestCase):
-    urls = 'middleware_exceptions.urls'
+@override_settings(ROOT_URLCONF='middleware_exceptions.urls')
+class RootUrlconfTests(SimpleTestCase):
 
     @override_settings(ROOT_URLCONF=None)
     def test_missing_root_urlconf(self):
@@ -788,3 +863,65 @@ class RootUrlconfTests(TestCase):
         # the previously defined settings.
         del settings.ROOT_URLCONF
         self.assertRaises(AttributeError, self.client.get, "/middleware_exceptions/view/")
+
+
+class MyMiddleware(object):
+
+    def __init__(self):
+        raise MiddlewareNotUsed
+
+    def process_request(self, request):
+        pass
+
+
+class MyMiddlewareWithExceptionMessage(object):
+
+    def __init__(self):
+        raise MiddlewareNotUsed('spam eggs')
+
+    def process_request(self, request):
+        pass
+
+
+@override_settings(
+    DEBUG=True,
+    ROOT_URLCONF='middleware_exceptions.urls',
+)
+class MiddlewareNotUsedTests(SimpleTestCase):
+
+    rf = RequestFactory()
+
+    def test_raise_exception(self):
+        request = self.rf.get('middleware_exceptions/view/')
+        with self.assertRaises(MiddlewareNotUsed):
+            MyMiddleware().process_request(request)
+
+    @override_settings(MIDDLEWARE_CLASSES=[
+        'middleware_exceptions.tests.MyMiddleware',
+    ])
+    def test_log(self):
+        with patch_logger('django.request', 'debug') as calls:
+            self.client.get('/middleware_exceptions/view/')
+        self.assertEqual(len(calls), 1)
+        self.assertEqual(
+            calls[0],
+            "MiddlewareNotUsed: 'middleware_exceptions.tests.MyMiddleware'"
+        )
+
+    @override_settings(MIDDLEWARE_CLASSES=[
+        'middleware_exceptions.tests.MyMiddlewareWithExceptionMessage',
+    ])
+    def test_log_custom_message(self):
+        with patch_logger('django.request', 'debug') as calls:
+            self.client.get('/middleware_exceptions/view/')
+        self.assertEqual(len(calls), 1)
+        self.assertEqual(
+            calls[0],
+            "MiddlewareNotUsed('middleware_exceptions.tests.MyMiddlewareWithExceptionMessage'): spam eggs"
+        )
+
+    @override_settings(DEBUG=False)
+    def test_do_not_log_when_debug_is_false(self):
+        with patch_logger('django.request', 'debug') as calls:
+            self.client.get('/middleware_exceptions/view/')
+        self.assertEqual(len(calls), 0)
