@@ -2,9 +2,10 @@ from __future__ import unicode_literals
 
 from operator import attrgetter
 
+from django.apps.registry import Apps
 from django.core.exceptions import FieldError, ValidationError
 from django.core.management import call_command
-from django.db import connection
+from django.db import connection, models
 from django.test import TestCase, TransactionTestCase
 from django.test.utils import CaptureQueriesContext
 from django.utils import six
@@ -130,6 +131,26 @@ class ModelInheritanceTests(TestCase):
         m = MixinModel()
         self.assertEqual(m.other_attr, 1)
 
+    def test_abstract_parent_link(self):
+        test_apps = Apps(['model_inheritance'])
+
+        class A(models.Model):
+            class Meta:
+                apps = test_apps
+
+        class B(A):
+            a = models.OneToOneField('A', parent_link=True, on_delete=models.CASCADE)
+
+            class Meta:
+                apps = test_apps
+                abstract = True
+
+        class C(B):
+            class Meta:
+                apps = test_apps
+
+        self.assertIs(C._meta.parents[A], C._meta.get_field('a'))
+
 
 class ModelInheritanceDataTests(TestCase):
     @classmethod
@@ -403,7 +424,7 @@ class InheritanceSameModelNameTests(TransactionTestCase):
     def test_inheritance_with_same_model_name(self):
         with self.modify_settings(
                 INSTALLED_APPS={'append': ['model_inheritance.same_model_name']}):
-            call_command('migrate', verbosity=0)
+            call_command('migrate', verbosity=0, run_syncdb=True)
             from .same_model_name.models import Copy
             copy = self.title.attached_same_model_name_copy_set.create(
                 content='The Web framework for perfectionists with deadlines.',
