@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-39. Testing using the Test Client
+Testing using the Test Client
 
 The test client is a class that can act like a simple
 browser for testing purposes.
@@ -22,17 +22,43 @@ rather than the HTML rendered to the end-user.
 """
 from __future__ import unicode_literals
 
+import datetime
+
+from django.contrib.auth.models import User
 from django.core import mail
-from django.test import Client, TestCase, RequestFactory
-from django.test import override_settings
+from django.core.urlresolvers import reverse_lazy
+from django.http import HttpResponse
+from django.test import (
+    Client, RequestFactory, SimpleTestCase, TestCase, override_settings,
+)
 
-from .views import get_view
+from .views import get_view, post_view, trace_view
 
 
-@override_settings(PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
+@override_settings(PASSWORD_HASHERS=['django.contrib.auth.hashers.SHA1PasswordHasher'],
+                   ROOT_URLCONF='test_client.urls',)
 class ClientTest(TestCase):
-    fixtures = ['testdata.json']
-    urls = 'test_client.urls'
+
+    @classmethod
+    def setUpTestData(cls):
+        cls.u1 = User.objects.create(
+            password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
+            last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='testclient',
+            first_name='Test', last_name='Client', email='testclient@example.com', is_staff=False, is_active=True,
+            date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
+        )
+        cls.u2 = User.objects.create(
+            password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
+            last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='inactive',
+            first_name='Inactive', last_name='User', email='testclient@example.com', is_staff=False, is_active=False,
+            date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
+        )
+        cls.u3 = User.objects.create(
+            password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
+            last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='staff',
+            first_name='Staff', last_name='Member', email='testclient@example.com', is_staff=True, is_active=True,
+            date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
+        )
 
     def test_get_view(self):
         "GET a view"
@@ -79,6 +105,13 @@ class ClientTest(TestCase):
         self.assertEqual(response.templates[0].name, 'POST Template')
         self.assertContains(response, 'Data received')
 
+    def test_trace(self):
+        """TRACE a view"""
+        response = self.client.trace('/trace_view/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['method'], 'TRACE')
+        self.assertEqual(response.templates[0].name, 'TRACE Template')
+
     def test_response_headers(self):
         "Check the value of HTTP headers returned in a response"
         response = self.client.get("/header_view/")
@@ -99,9 +132,34 @@ class ClientTest(TestCase):
             self.assertIn(key, response.wsgi_request.environ)
             self.assertEqual(response.wsgi_request.environ[key], value)
 
+    def test_response_resolver_match(self):
+        """
+        The response contains a ResolverMatch instance.
+        """
+        response = self.client.get('/header_view/')
+        self.assertTrue(hasattr(response, 'resolver_match'))
+
+    def test_response_resolver_match_redirect_follow(self):
+        """
+        The response ResolverMatch instance contains the correct
+        information when following redirects.
+        """
+        response = self.client.get('/redirect_view/', follow=True)
+        self.assertEqual(response.resolver_match.url_name, 'get_view')
+
+    def test_response_resolver_match_regular_view(self):
+        """
+        The response ResolverMatch instance contains the correct
+        information when accessing a regular view.
+        """
+        response = self.client.get('/get_view/')
+        self.assertEqual(response.resolver_match.url_name, 'get_view')
+
     def test_raw_post(self):
         "POST raw data (with a content type) to a view"
-        test_doc = """<?xml version="1.0" encoding="utf-8"?><library><book><title>Blink</title><author>Malcolm Gladwell</author></book></library>"""
+        test_doc = """<?xml version="1.0" encoding="utf-8"?>
+        <library><book><title>Blink</title><author>Malcolm Gladwell</author></book></library>
+        """
         response = self.client.post("/raw_post_view/", test_doc,
                                     content_type="text/xml")
         self.assertEqual(response.status_code, 200)
@@ -123,40 +181,27 @@ class ClientTest(TestCase):
     def test_redirect(self):
         "GET a URL that redirects elsewhere"
         response = self.client.get('/redirect_view/')
-        # Check that the response was a 302 (redirect) and that
-        # assertRedirect() understands to put an implicit http://testserver/ in
-        # front of non-absolute URLs.
+        # Check that the response was a 302 (redirect)
         self.assertRedirects(response, '/get_view/')
 
-        host = 'django.testserver'
-        client_providing_host = Client(HTTP_HOST=host)
-        response = client_providing_host.get('/redirect_view/')
-        # Check that the response was a 302 (redirect) with absolute URI
-        self.assertRedirects(response, '/get_view/', host=host)
-
     def test_redirect_with_query(self):
         "GET a URL that redirects with given GET parameters"
         response = self.client.get('/redirect_view/', {'var': 'value'})
 
         # Check if parameters are intact
-        self.assertRedirects(response, 'http://testserver/get_view/?var=value')
+        self.assertRedirects(response, '/get_view/?var=value')
 
     def test_permanent_redirect(self):
         "GET a URL that redirects permanently elsewhere"
         response = self.client.get('/permanent_redirect_view/')
         # Check that the response was a 301 (permanent redirect)
-        self.assertRedirects(response, 'http://testserver/get_view/', status_code=301)
-
-        client_providing_host = Client(HTTP_HOST='django.testserver')
-        response = client_providing_host.get('/permanent_redirect_view/')
-        # Check that the response was a 301 (permanent redirect) with absolute URI
-        self.assertRedirects(response, 'http://django.testserver/get_view/', status_code=301)
+        self.assertRedirects(response, '/get_view/', status_code=301)
 
     def test_temporary_redirect(self):
         "GET a URL that does a non-permanent redirect"
         response = self.client.get('/temporary_redirect_view/')
         # Check that the response was a 302 (non-permanent redirect)
-        self.assertRedirects(response, 'http://testserver/get_view/', status_code=302)
+        self.assertRedirects(response, '/get_view/', status_code=302)
 
     def test_redirect_to_strange_location(self):
         "GET a URL that redirects to a non-200 page"
@@ -164,14 +209,26 @@ class ClientTest(TestCase):
 
         # Check that the response was a 302, and that
         # the attempt to get the redirection location returned 301 when retrieved
-        self.assertRedirects(response, 'http://testserver/permanent_redirect_view/', target_status_code=301)
+        self.assertRedirects(response, '/permanent_redirect_view/', target_status_code=301)
 
     def test_follow_redirect(self):
         "A URL that redirects can be followed to termination."
         response = self.client.get('/double_redirect_view/', follow=True)
-        self.assertRedirects(response, 'http://testserver/get_view/', status_code=302, target_status_code=200)
+        self.assertRedirects(response, '/get_view/', status_code=302, target_status_code=200)
         self.assertEqual(len(response.redirect_chain), 2)
 
+    def test_follow_relative_redirect(self):
+        "A URL with a relative redirect can be followed."
+        response = self.client.get('/accounts/', follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.request['PATH_INFO'], '/accounts/login/')
+
+    def test_follow_relative_redirect_no_trailing_slash(self):
+        "A URL with a relative redirect with no trailing slash can be followed."
+        response = self.client.get('/accounts/no_trailing_slash', follow=True)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.request['PATH_INFO'], '/accounts/login/')
+
     def test_redirect_http(self):
         "GET a URL that redirects to an http URI"
         response = self.client.get('/http_redirect_view/', follow=True)
@@ -311,7 +368,7 @@ class ClientTest(TestCase):
 
         # Get the page without logging in. Should result in 302.
         response = self.client.get('/login_protected_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/login_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_view/')
 
         # Log in
         login = self.client.login(username='testclient', password='password')
@@ -322,12 +379,26 @@ class ClientTest(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.context['user'].username, 'testclient')
 
+    def test_view_with_force_login(self):
+        "Request a page that is protected with @login_required"
+        # Get the page without logging in. Should result in 302.
+        response = self.client.get('/login_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_view/')
+
+        # Log in
+        self.client.force_login(self.u1)
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_view/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['user'].username, 'testclient')
+
     def test_view_with_method_login(self):
         "Request a page that is protected with a @login_required method"
 
         # Get the page without logging in. Should result in 302.
         response = self.client.get('/login_protected_method_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/login_protected_method_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_method_view/')
 
         # Log in
         login = self.client.login(username='testclient', password='password')
@@ -338,12 +409,26 @@ class ClientTest(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.context['user'].username, 'testclient')
 
+    def test_view_with_method_force_login(self):
+        "Request a page that is protected with a @login_required method"
+        # Get the page without logging in. Should result in 302.
+        response = self.client.get('/login_protected_method_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_method_view/')
+
+        # Log in
+        self.client.force_login(self.u1)
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_method_view/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['user'].username, 'testclient')
+
     def test_view_with_login_and_custom_redirect(self):
         "Request a page that is protected with @login_required(redirect_field_name='redirect_to')"
 
         # Get the page without logging in. Should result in 302.
         response = self.client.get('/login_protected_view_custom_redirect/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?redirect_to=/login_protected_view_custom_redirect/')
+        self.assertRedirects(response, '/accounts/login/?redirect_to=/login_protected_view_custom_redirect/')
 
         # Log in
         login = self.client.login(username='testclient', password='password')
@@ -354,6 +439,23 @@ class ClientTest(TestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.context['user'].username, 'testclient')
 
+    def test_view_with_force_login_and_custom_redirect(self):
+        """
+        Request a page that is protected with
+        @login_required(redirect_field_name='redirect_to')
+        """
+        # Get the page without logging in. Should result in 302.
+        response = self.client.get('/login_protected_view_custom_redirect/')
+        self.assertRedirects(response, '/accounts/login/?redirect_to=/login_protected_view_custom_redirect/')
+
+        # Log in
+        self.client.force_login(self.u1)
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_view_custom_redirect/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['user'].username, 'testclient')
+
     def test_view_with_bad_login(self):
         "Request a page that is protected with @login, but use bad credentials"
 
@@ -366,6 +468,21 @@ class ClientTest(TestCase):
         login = self.client.login(username='inactive', password='password')
         self.assertFalse(login)
 
+    def test_view_with_inactive_force_login(self):
+        "Request a page that is protected with @login, but use an inactive login"
+
+        # Get the page without logging in. Should result in 302.
+        response = self.client.get('/login_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_view/')
+
+        # Log in
+        self.client.force_login(self.u2)
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_view/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['user'].username, 'inactive')
+
     def test_logout(self):
         "Request a logout after logging in"
         # Log in
@@ -381,7 +498,48 @@ class ClientTest(TestCase):
 
         # Request a page that requires a login
         response = self.client.get('/login_protected_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/login_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_view/')
+
+    def test_logout_with_force_login(self):
+        "Request a logout after logging in"
+        # Log in
+        self.client.force_login(self.u1)
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_view/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['user'].username, 'testclient')
+
+        # Log out
+        self.client.logout()
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_view/')
+
+    @override_settings(
+        AUTHENTICATION_BACKENDS=[
+            'django.contrib.auth.backends.ModelBackend',
+            'test_client.auth_backends.TestClientBackend',
+        ],
+    )
+    def test_force_login_with_backend(self):
+        """
+        Request a page that is protected with @login_required when using
+        force_login() and passing a backend.
+        """
+        # Get the page without logging in. Should result in 302.
+        response = self.client.get('/login_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/login_protected_view/')
+
+        # Log in
+        self.client.force_login(self.u1, backend='test_client.auth_backends.TestClientBackend')
+        self.assertEqual(self.u1.backend, 'test_client.auth_backends.TestClientBackend')
+
+        # Request a page that requires a login
+        response = self.client.get('/login_protected_view/')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response.context['user'].username, 'testclient')
 
     @override_settings(SESSION_ENGINE="django.contrib.sessions.backends.signed_cookies")
     def test_logout_cookie_sessions(self):
@@ -392,7 +550,7 @@ class ClientTest(TestCase):
 
         # Get the page without logging in. Should result in 302.
         response = self.client.get('/permission_protected_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/permission_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/permission_protected_view/')
 
         # Log in
         login = self.client.login(username='testclient', password='password')
@@ -400,7 +558,7 @@ class ClientTest(TestCase):
 
         # Log in with wrong permissions. Should result in 302.
         response = self.client.get('/permission_protected_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/permission_protected_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/permission_protected_view/')
 
         # TODO: Log in with right permissions and request the page again
 
@@ -424,7 +582,7 @@ class ClientTest(TestCase):
 
         # Get the page without logging in. Should result in 302.
         response = self.client.get('/permission_protected_method_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/permission_protected_method_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/permission_protected_method_view/')
 
         # Log in
         login = self.client.login(username='testclient', password='password')
@@ -432,7 +590,7 @@ class ClientTest(TestCase):
 
         # Log in with wrong permissions. Should result in 302.
         response = self.client.get('/permission_protected_method_view/')
-        self.assertRedirects(response, 'http://testserver/accounts/login/?next=/permission_protected_method_view/')
+        self.assertRedirects(response, '/accounts/login/?next=/permission_protected_method_view/')
 
         # TODO: Log in with right permissions and request the page again
 
@@ -478,6 +636,22 @@ class ClientTest(TestCase):
         self.assertEqual(mail.outbox[0].to[0], 'first@example.com')
         self.assertEqual(mail.outbox[0].to[1], 'second@example.com')
 
+    def test_reverse_lazy_decodes(self):
+        "Ensure reverse_lazy works in the test client"
+        data = {'var': 'data'}
+        response = self.client.get(reverse_lazy('get_view'), data)
+
+        # Check some response details
+        self.assertContains(response, 'This is a test')
+
+    def test_relative_redirect(self):
+        response = self.client.get('/accounts/')
+        self.assertRedirects(response, '/accounts/login/')
+
+    def test_relative_redirect_no_trailing_slash(self):
+        response = self.client.get('/accounts/no_trailing_slash')
+        self.assertRedirects(response, '/accounts/login/')
+
     def test_mass_mail_sending(self):
         "Test that mass mail is redirected to a dummy outbox during test setup"
 
@@ -497,12 +671,20 @@ class ClientTest(TestCase):
         self.assertEqual(mail.outbox[1].to[0], 'second@example.com')
         self.assertEqual(mail.outbox[1].to[1], 'third@example.com')
 
+    def test_exception_following_nested_client_request(self):
+        """
+        A nested test client request shouldn't clobber exception signals from
+        the outer client request.
+        """
+        with self.assertRaisesMessage(Exception, 'exception message'):
+            self.client.get('/nesting_exception_view/')
+
 
 @override_settings(
-    MIDDLEWARE_CLASSES=('django.middleware.csrf.CsrfViewMiddleware',)
+    MIDDLEWARE_CLASSES=['django.middleware.csrf.CsrfViewMiddleware'],
+    ROOT_URLCONF='test_client.urls',
 )
-class CSRFEnabledClientTests(TestCase):
-    urls = 'test_client.urls'
+class CSRFEnabledClientTests(SimpleTestCase):
 
     def test_csrf_enabled_client(self):
         "A client can be instantiated with CSRF checks enabled"
@@ -521,7 +703,7 @@ class CustomTestClient(Client):
     i_am_customized = "Yes"
 
 
-class CustomTestClientTest(TestCase):
+class CustomTestClientTest(SimpleTestCase):
     client_class = CustomTestClient
 
     def test_custom_test_client(self):
@@ -529,13 +711,54 @@ class CustomTestClientTest(TestCase):
         self.assertEqual(hasattr(self.client, "i_am_customized"), True)
 
 
-class RequestFactoryTest(TestCase):
-    urls = 'test_client.urls'
+_generic_view = lambda request: HttpResponse(status=200)
+
+
+@override_settings(ROOT_URLCONF='test_client.urls')
+class RequestFactoryTest(SimpleTestCase):
+    """Tests for the request factory."""
+
+    # A mapping between names of HTTP/1.1 methods and their test views.
+    http_methods_and_views = (
+        ('get', get_view),
+        ('post', post_view),
+        ('put', _generic_view),
+        ('patch', _generic_view),
+        ('delete', _generic_view),
+        ('head', _generic_view),
+        ('options', _generic_view),
+        ('trace', trace_view),
+    )
+
+    def setUp(self):
+        self.request_factory = RequestFactory()
 
     def test_request_factory(self):
-        factory = RequestFactory()
-        request = factory.get('/somewhere/')
+        """The request factory implements all the HTTP/1.1 methods."""
+        for method_name, view in self.http_methods_and_views:
+            method = getattr(self.request_factory, method_name)
+            request = method('/somewhere/')
+            response = view(request)
+
+            self.assertEqual(response.status_code, 200)
+
+    def test_get_request_from_factory(self):
+        """
+        The request factory returns a templated response for a GET request.
+        """
+        request = self.request_factory.get('/somewhere/')
         response = get_view(request)
 
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, 'This is a test')
+
+    def test_trace_request_from_factory(self):
+        """The request factory returns an echo response for a TRACE request."""
+        url_path = '/somewhere/'
+        request = self.request_factory.trace(url_path)
+        response = trace_view(request)
+        protocol = request.META["SERVER_PROTOCOL"]
+        echoed_request_line = "TRACE {} {}".format(url_path, protocol)
+
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, echoed_request_line)
