@@ -16,9 +16,19 @@ class ModelToValidate(models.Model):
     name = models.CharField(max_length=100)
     created = models.DateTimeField(default=datetime.now)
     number = models.IntegerField(db_column='number_val')
-    parent = models.ForeignKey('self', blank=True, null=True, limit_choices_to={'number': 10})
+    parent = models.ForeignKey(
+        'self',
+        models.SET_NULL,
+        blank=True, null=True,
+        limit_choices_to={'number': 10},
+    )
     email = models.EmailField(blank=True)
-    ufm = models.ForeignKey('UniqueFieldsModel', to_field='unique_charfield', blank=True, null=True)
+    ufm = models.ForeignKey(
+        'UniqueFieldsModel',
+        models.SET_NULL,
+        to_field='unique_charfield',
+        blank=True, null=True,
+    )
     url = models.URLField(blank=True)
     f_with_custom_validator = models.IntegerField(blank=True, null=True, validators=[validate_answer_to_universe])
     slug = models.SlugField(blank=True)
@@ -70,7 +80,7 @@ class Author(models.Model):
 
 class Article(models.Model):
     title = models.CharField(max_length=100)
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     pub_date = models.DateTimeField(blank=True)
 
     def clean(self):
