@@ -10,11 +10,12 @@ from unittest import TestCase
 
 from django.core.exceptions import ValidationError
 from django.core.validators import (
-    BaseValidator, EmailValidator, MaxLengthValidator, MaxValueValidator,
-    MinLengthValidator, MinValueValidator, RegexValidator, URLValidator,
-    validate_comma_separated_integer_list, validate_email, validate_integer,
-    validate_ipv4_address, validate_ipv6_address, validate_ipv46_address,
-    validate_slug,
+    BaseValidator, DecimalValidator, EmailValidator, MaxLengthValidator,
+    MaxValueValidator, MinLengthValidator, MinValueValidator, RegexValidator,
+    URLValidator, int_list_validator, validate_comma_separated_integer_list,
+    validate_email, validate_integer, validate_ipv4_address,
+    validate_ipv6_address, validate_ipv46_address, validate_slug,
+    validate_unicode_slug,
 )
 from django.test import SimpleTestCase
 from django.test.utils import str_prefix
@@ -47,7 +48,12 @@ TEST_DATA = [
     (validate_email, 'email@localhost', None),
     (EmailValidator(whitelist=['localdomain']), 'email@localdomain', None),
     (validate_email, '"test@test"@example.com', None),
+    (validate_email, 'example@atm.%s' % ('a' * 63), None),
+    (validate_email, 'example@%s.atm' % ('a' * 63), None),
+    (validate_email, 'example@%s.%s.atm' % ('a' * 63, 'b' * 10), None),
 
+    (validate_email, 'example@atm.%s' % ('a' * 64), ValidationError),
+    (validate_email, 'example@%s.atm.%s' % ('b' * 64, 'a' * 63), ValidationError),
     (validate_email, None, ValidationError),
     (validate_email, '', ValidationError),
     (validate_email, 'abc', ValidationError),
@@ -71,9 +77,9 @@ TEST_DATA = [
     (validate_email, '"\\\011"@here.com', None),
     (validate_email, '"\\\012"@here.com', ValidationError),
     (validate_email, 'trailingdot@shouldfail.com.', ValidationError),
-    # Max length of domain name in email is 249 (see validator for calculation)
-    (validate_email, 'a@%s.us' % ('a' * 249), None),
-    (validate_email, 'a@%s.us' % ('a' * 250), ValidationError),
+    # Max length of domain name labels is 63 characters per RFC 1034.
+    (validate_email, 'a@%s.us' % ('a' * 63), None),
+    (validate_email, 'a@%s.us' % ('a' * 64), ValidationError),
     # Trailing newlines in username or domain not allowed
     (validate_email, 'a@b.com\n', ValidationError),
     (validate_email, 'a\n@b.com', ValidationError),
@@ -84,15 +90,36 @@ TEST_DATA = [
     (validate_slug, 'longer-slug-still-ok', None),
     (validate_slug, '--------', None),
     (validate_slug, 'nohyphensoranything', None),
+    (validate_slug, 'a', None),
+    (validate_slug, '1', None),
+    (validate_slug, 'a1', None),
 
     (validate_slug, '', ValidationError),
     (validate_slug, ' text ', ValidationError),
     (validate_slug, ' ', ValidationError),
     (validate_slug, 'some@mail.com', ValidationError),
     (validate_slug, '你好', ValidationError),
+    (validate_slug, '你 好', ValidationError),
     (validate_slug, '\n', ValidationError),
     (validate_slug, 'trailing-newline\n', ValidationError),
 
+    (validate_unicode_slug, 'slug-ok', None),
+    (validate_unicode_slug, 'longer-slug-still-ok', None),
+    (validate_unicode_slug, '--------', None),
+    (validate_unicode_slug, 'nohyphensoranything', None),
+    (validate_unicode_slug, 'a', None),
+    (validate_unicode_slug, '1', None),
+    (validate_unicode_slug, 'a1', None),
+    (validate_unicode_slug, '你好', None),
+
+    (validate_unicode_slug, '', ValidationError),
+    (validate_unicode_slug, ' text ', ValidationError),
+    (validate_unicode_slug, ' ', ValidationError),
+    (validate_unicode_slug, 'some@mail.com', ValidationError),
+    (validate_unicode_slug, '\n', ValidationError),
+    (validate_unicode_slug, '你 好', ValidationError),
+    (validate_unicode_slug, 'trailing-newline\n', ValidationError),
+
     (validate_ipv4_address, '1.1.1.1', None),
     (validate_ipv4_address, '255.0.0.0', None),
     (validate_ipv4_address, '0.0.0.0', None),
@@ -129,13 +156,24 @@ TEST_DATA = [
     (validate_ipv46_address, '12345::', ValidationError),
 
     (validate_comma_separated_integer_list, '1', None),
+    (validate_comma_separated_integer_list, '12', None),
+    (validate_comma_separated_integer_list, '1,2', None),
     (validate_comma_separated_integer_list, '1,2,3', None),
-    (validate_comma_separated_integer_list, '1,2,3,', None),
+    (validate_comma_separated_integer_list, '10,32', None),
 
     (validate_comma_separated_integer_list, '', ValidationError),
+    (validate_comma_separated_integer_list, 'a', ValidationError),
     (validate_comma_separated_integer_list, 'a,b,c', ValidationError),
     (validate_comma_separated_integer_list, '1, 2, 3', ValidationError),
-    (validate_comma_separated_integer_list, '1,2,3\n', ValidationError),
+    (validate_comma_separated_integer_list, ',', ValidationError),
+    (validate_comma_separated_integer_list, '1,2,3,', ValidationError),
+    (validate_comma_separated_integer_list, '1,2,', ValidationError),
+    (validate_comma_separated_integer_list, ',1', ValidationError),
+    (validate_comma_separated_integer_list, '1,,2', ValidationError),
+
+    (int_list_validator(sep='.'), '1.2.3', None),
+    (int_list_validator(sep='.'), '1,2,3', ValidationError),
+    (int_list_validator(sep='.'), '1.2.3\n', ValidationError),
 
     (MaxValueValidator(10), 10, None),
     (MaxValueValidator(10), -10, None),
@@ -364,3 +402,21 @@ class TestValidatorEquality(TestCase):
             MinValueValidator(45),
             MinValueValidator(11),
         )
+
+    def test_decimal_equality(self):
+        self.assertEqual(
+            DecimalValidator(1, 2),
+            DecimalValidator(1, 2),
+        )
+        self.assertNotEqual(
+            DecimalValidator(1, 2),
+            DecimalValidator(1, 1),
+        )
+        self.assertNotEqual(
+            DecimalValidator(1, 2),
+            DecimalValidator(2, 2),
+        )
+        self.assertNotEqual(
+            DecimalValidator(1, 2),
+            MinValueValidator(11),
+        )
